<?php

/**
 * VTB Payments main handler class
 *
 * @copyright  VTB
 * @author     VTB
 * @version    1.0.1
 *
 */

require_once 'vendor/autoload.php';

use \Vtbpay\Classes\VtbApi,
    \Vtbpay\Classes\VtbPayLogger;

class VtbPayments extends \Simpla
{
    /**
     * @var \Simpla Current order
     */
    private $order;
    
    /**
     * @var mixed Selected payment method
     */
    private $paymentMethod;
    
    /**
     * @var array Payment system settings
     */
    private $paymentSettings; // Настройки
    
    /**
     * @var \Vtbpay\Classes\VtbPayLogger Logging class
     */
    private $logger; 
    
    
    /**
     * Create and return an instance of the VtbApi class for VtbPayments integration.
     *
     * @return VtbApi An instance of the VtbApi class.
     */
    private function getVtbApi()
    {
       $settings = $this->paymentSettings;
       
       return new VtbApi(
           $settings['vtbpay_client_id'],
           $settings['vtbpay_client_secret'],
           $settings['vtbpay_test_mode'],
           $settings['vtbpay_merchant_authorization']
       );
    }
    
    
    /**
     * Generates the checkout form for VtbPay payment.
     *
     * @param int $orderId The ID of the order for which the checkout form is generated.
     * @param string|null $buttonText The text to display on the checkout button (optional).
     * @return string The HTML representation of the checkout button with VtbPay payment link.
     * @throws \Exception Throws an exception if there are errors during the checkout process.
     */
    public function checkout_form($orderId, $buttonText = null)
    {
        if (empty($buttonText)) {
            $buttonText = 'Перейти к оплате';
        }
        
        $this->logger = new VtbPayLogger(__DIR__ . '/log/VtbPayments.log');
        
        try {
            $this->checkOrder($orderId);

            $this->checkPaymentMethod();
            
            $this->setPaymentSettings();
            
            $this->checkCurrency();
            
            // Check if the payment URL is present and not empty in the response.
            if (empty($payUrl = $this->getPayUrl())) {
                // Throw an exception if the payment URL is missing or empty.
                throw new \Exception('Не удалось получить URL-адрес оплаты.');
            }
            
            return $this->generateRedirectHtml($payUrl, $buttonText);
                    
        } catch (\Exception $e) {
            // Log the caught exception for debugging purposes.
            $this->logger->log(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: (%s)', 
                $e->getMessage(), 
                $orderId ?: ''
            ), VtbPayLogger::LOG_LEVEL_ERROR, [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ]);
            
            return '<span style="color:red;">' . $e->getMessage() . '</span>';
        }
    }
    
    
    /**
     * Generates HTML code for automatic redirection with a delay.
     *
     * @param string $payUrl The URL for redirection.
     * @param string $buttonText The text to display on the button.
     * @return string The generated HTML code.
     */
    private function generateRedirectHtml($payUrl, $buttonText)
    {
        return "<div>
                    <p>
                      Страница будет автоматически переадресована на платежную форму через 3 секунды.<br>
                      Нажмите на ссылку ниже для незамедлительного перехода.
                    </p>
                    <a href=\"{$payUrl}\" class=\"checkout_button\" id=\"vtbpay-button-confirm\">{$buttonText}</a>
                </div>
                <script type=\"text/javascript\">
                    const paymentButton = document.getElementById(\"vtbpay-button-confirm\");
                    if (paymentButton) {
                        setTimeout(function () {
                            paymentButton.click();
                        }, 3000); // 3000 миллисекунд = 3 секунды
                    }
                </script>";
    }
    
    
    /**
     * Checks the existence of an order.
     *
     * @param int $orderId The ID of the order to check.
     * @throws \Exception Throws an exception if the order does not exist.
     */
    private function checkOrder($orderId)
    {
        // Существует ли заказ
        $this->order = $this->orders->get_order((int)$orderId);
        
        if (empty($this->order)) {
            throw new \Exception('Оплачиваемый заказ не найден');
        }
    }
    

    /**
     * Checks the validity of the payment method.
     *
     * @throws \Exception Throws an exception if the payment method is invalid.
     */
    private function checkPaymentMethod()
    {
        // Наш ли способ оплаты
        $this->paymentMethod = $this->payment->get_payment_method(intval($this->order->payment_method_id));
        
        if (empty($this->paymentMethod)) {
            throw new \Exception('Неизвестный метод оплаты');
        }
    }
    
    
    /**
     * Sets the payment settings.
     *
     * @throws \Exception Throws an exception if the payment settings are missing.
     */
    private function setPaymentSettings()
    {
        $this->paymentSettings = $this->payment->get_payment_settings($this->paymentMethod->id);
        
        if (empty($this->paymentSettings)) {
            throw new \Exception('Настройки платёжной системы не указаны.');
        }
    }
    
    
    /**
     * Checks the currency for payment eligibility.
     *
     * @throws \Exception Throws an exception if payment is not possible in the specified currency.
     */
    private function checkCurrency()
    {
        $paymentCurrency = $this->money->get_currency(intval($this->paymentMethod->currency_id));
        
        if ($paymentCurrency->code !== 'RUB') {
            throw new \Exception('Оплата возможна только в рублях.');
        }
    }
    
    
    /**
     * Retrieves the payment URL for redirection.
     *
     * @return string The payment URL.
     * @throws \Exception Throws an exception if the payment URL cannot be obtained.
     */
    private function getPayUrl()
    {
        // Receive the final payment amount
        $amount = round($this->money->convert(
            $this->order->total_price, 
            $this->paymentMethod->currency_id, 
            false
        ), 2);
        
        // Return page after successful/unsuccessful payment
        $returnUrl = $this->config->root_url . '/payment/' . __CLASS__ . '/callback.php';
      
        $response = $this->getVtbApi()->getOrderLink(
            $this->order->id,
            $this->order->email,
            time(),
            $amount,
            $returnUrl
        );
        
        // Log the request and response data for debugging, if logging is enabled.
        if($this->paymentSettings['vtbpay_logging']) $this->logger->log(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderLink', 
            VtbPayLogger::LOG_LEVEL_DEBUG, [
            'arguments' => [
                'orderId' => $this->order->id, 
                'email' => $this->order->email, 
                'date' => time(), 
                'amount' => $amount, 
                'returnUrl' => $returnUrl
            ],
            'response' => $response
        ]);
        
        return $response['object']['payUrl'] ?? '';
    }
}